using System;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace BasicEditor
{

    public class Camera : ICloneable
    {

        /// <summary>
        ///		Camera orientation.
        /// </summary>
        protected Vector3 orientation;
        /// <summary>
        ///		Camera position.
        /// </summary>
        protected Vector3 position;
        /// <summary>
        ///		Camera direction.
        /// </summary>
        protected Vector3 direction;
        /// <summary>
        ///		Whether aspect ratio will automaticaally be recalculated when a vieport changes its size.
        /// </summary>
        protected bool autoAspectRatio=true;
        /// <summary>
        ///     y-direction field-of-view (default 45).
        /// </summary>
        protected float fieldOfView;
        /// <summary>
        ///     Far clip distance - default 10000.
        /// </summary>
        protected float farDistance;
        /// <summary>
        ///     Near clip distance - default 100.
        /// </summary>
        protected float nearDistance;
        /// <summary>
        ///     x/y viewport ratio - default 1.3333
        /// </summary>
        protected float aspectRatio;
        /// <summary>
        ///     Pre-calced projection matrix.
        /// </summary>
        protected Matrix projectionMatrix;
        /// <summary>
        ///     Pre-calced view matrix.
        /// </summary>
        protected Matrix viewMatrix;
        /// <summary>
        ///     Something in the view/projection  has changed?
        /// </summary>
        protected bool needsUpdate = true;
        /// <summary>
        ///		Perspective or Orthographic?
        /// </summary>
        protected Projection projectionType;
        private Viewport viewport;
        protected bool viewportChanged = false;



        public float AspectRatio
        {
            get { return this.aspectRatio; }
            set
            {
                this.aspectRatio = value;
                this.needsUpdate = true;
            }
        }

        public Projection ProjectionType
        {
            get { return this.projectionType; }
            set { this.projectionType = value; }
        }

        public float NearDistance
        {
            get { return this.nearDistance; }
            set
            {
                this.nearDistance = value;
                this.needsUpdate = true;
            }
        }

        public float FarDistance
        {
            get { return this.farDistance; }
            set
            {
                this.farDistance = value;
                this.needsUpdate = true;
            }
        }

        public float FieldOfView
        {
            get { return this.fieldOfView; }
            set
            {
                this.fieldOfView = value;
                this.needsUpdate = true;
            }
        }

        public Viewport Viewport
        {
            get { return viewport; }
            set
            {
                viewport = value;
                viewportChanged = true;
                if (this.autoAspectRatio)
                {
                    aspectRatio = this.viewport.Width / this.viewport.Height;
                }
            }
        }

        public int ViewportX
        {
            get { return viewport.X; }
            set
            {
                viewport.X = value;
                viewportChanged = true;
            }
        }

        public int ViewportY
        {
            get { return viewport.Y; }
            set
            {
                viewport.Y = value;
                viewportChanged = true;
            }
        }

        public int ViewportWidth
        {
            get { return viewport.Width; }
            set
            {
                viewport.Width = value;
                viewportChanged = true;
                if (this.autoAspectRatio)
                {
                    aspectRatio = this.viewport.Width / this.viewport.Height;
                }
            }
        }

        public int ViewportHeight
        {
            get { return viewport.Height; }
            set
            {
                viewport.Height = value;
                viewportChanged = true;
                if (this.autoAspectRatio)
                {
                    aspectRatio = this.viewport.Width / this.viewport.Height;
                }
            }
        }

        public float ViewportMinDepth
        {
            get { return viewport.MinDepth; }
            set
            {
                viewport.MinDepth = value;
                viewportChanged = true;
            }
        }

        public float ViewportMaxDepth
        {
            get { return viewport.MaxDepth; }
            set
            {
                viewport.MaxDepth = value;
                viewportChanged = true;
            }
        }

        public Matrix ViewMatrix
        {
            get { return viewMatrix; }
        }

        public Matrix ProjectionMatrix
        {
            get { return projectionMatrix; }
        }

        public Camera()
        {
            // Init camera location & direction

            // Locate at (0,0,0)
            position = Vector3.Backward * 100;
            direction = Vector3.Zero;
            orientation = Vector3.Up;

            fieldOfView = MathHelper.PiOver2;
            nearDistance = 1.0f;
            farDistance = 10000.0f;
            aspectRatio = 1.33333333333333f;
            projectionType = Projection.Perspective;

            viewMatrix = Matrix.Identity;
            projectionMatrix = Matrix.Identity;
            this.Update();
        }


        /// <summary>
        ///		Signal to update view information.
        /// </summary>
        protected void InvalidateView()
        {
            needsUpdate = true;
        }

        public bool NeedsUpdate
        {
            get { return needsUpdate; }
        }

        public bool ProjectionNeedsUpdate
        {
            get { return needsUpdate; }
        }

        public bool ViewportChanged
        {
            get { return this.viewportChanged; }
            set { this.viewportChanged = value; }
        }

        public float LookDistance()
        {
            return Vector3.Distance(this.position, this.direction);
        }

        /// <summary>
        ///		Updates the view matrix.
        /// </summary>
        public void Update()
        {
            switch (this.projectionType)
            {
                case Projection.Orthographic:
                    // do nothing
                    // viewMatrix = Matrix.CreateOrthographic(this.viewport.Width, this.viewport.Height, this.nearDistance, this.farDistance);
                   // projectionMatrix = Matrix.CreateOrthographic(this.viewport.Width, this.viewport.Height, this.nearDistance, this.farDistance);
                    projectionMatrix = Matrix.CreateOrthographicOffCenter(this.viewport.X, this.viewport.Width, this.viewport.Height, this.viewport.Y, this.nearDistance, this.farDistance);
                    break;

                case Projection.Perspective:
                    viewMatrix = Matrix.CreateLookAt(this.position, this.direction, this.orientation);
                    projectionMatrix = Matrix.CreatePerspectiveFieldOfView(this.fieldOfView, this.aspectRatio, this.nearDistance, this.farDistance);
                    break;
            }

            this.needsUpdate = false;
        }

        /// <summary>
        ///		If set to true a vieport that owns this frustum will be able to 
        ///		recalculate the aspect ratio whenever the frustum is resized.
        /// </summary>
        /// <remarks>
        ///		You should set this to true only if the frustum / camera is used by 
        ///		one viewport at the same time. Otherwise the aspect ratio for other 
        ///		viewports may be wrong.
        /// </remarks>
        public bool AutoAspectRatio
        {
            get { return autoAspectRatio; }
            set { autoAspectRatio = value; }
        }

        /// <summary>
        ///     Gets/Sets the camera's orientation.
        /// </summary>
        public Vector3 Orientation
        {
            get { return orientation; }
            set
            {
                orientation = value;
                InvalidateView();
            }
        }

        /// <summary>
        ///     Gets/Sets the camera's position.
        /// </summary>
        public Vector3 Position
        {
            get { return position; }
            set
            {
                position = value;
                InvalidateView();
            }
        }

        /// <summary>
        ///		Gets/Sets the camera's direction vector.
        /// </summary>
        public Vector3 Direction
        {
            // Direction points down the negatize Z axis by default.
            //orientation * -Vector3.UnitZ;
            get { return direction; }
            set
            {
                direction = value;

                //// Do nothing if given a zero vector
                //// (Replaced assert since this could happen with auto tracking camera and
                ////  camera passes through the lookAt point)
                //if ( direction == Vector3.Zero )
                //    return;

                //// Remember, camera points down -Z of local axes!
                //// Therefore reverse direction of direction vector before determining local Z
                //Vector3 zAdjustVector = -direction;
                //zAdjustVector.Normalize();

                //if ( isYawFixed )
                //{
                //    Vector3 xVector = yawFixedAxis.Cross( zAdjustVector );
                //    xVector.Normalize();

                //    Vector3 yVector = zAdjustVector.Cross( xVector );
                //    yVector.Normalize();

                //    orientation.FromAxes( xVector, yVector, zAdjustVector );
                //}
                //else
                //{
                //    // update the view of the camera
                //    UpdateView();

                //    // Get axes from current quaternion
                //    Vector3 xAxis, yAxis, zAxis;

                //    // get the vector components of the derived orientation vector
                //    this.DerivedOrientation.ToAxes( out xAxis, out yAxis, out zAxis );

                //    Quaternion rotationQuat;

                //    if ( -zAdjustVector == zAxis )
                //    {
                //        // Oops, a 180 degree turn (infinite possible rotation axes)
                //        // Default to yaw i.e. use current UP
                //        rotationQuat = Quaternion.FromAngleAxis( Utility.PI, yAxis );
                //    }
                //    else
                //    {
                //        // Derive shortest arc to new direction
                //        rotationQuat = zAxis.GetRotationTo( zAdjustVector );
                //    }

                //    orientation = rotationQuat * orientation;
                //}

                // TODO: If we have a fixed yaw axis, we musn't break it by using the
                // shortest arc because this will sometimes cause a relative yaw
                // which will tip the camera

                InvalidateView();
            }
        }

        ///// <summary>
        /////		Gets camera's 'right' vector.
        ///// </summary>
        //public Vector3 Right
        //{
        //    get
        //    {
        //        return orientation * Vector3.UnitX;
        //    }
        //}

        ///// <summary>
        /////		Gets camera's 'up' vector.
        ///// </summary>
        //public Vector3 Up
        //{
        //    get
        //    {
        //        return orientation * Vector3.UnitY;
        //    }
        //}




        /// <summary>
        /// Moves the camera's position by the vector offset provided along world axes.
        /// </summary>
        /// <param name="offset"></param>
        public void Move(Vector3 offset)
        {
            position = position + offset;
            InvalidateView();
        }

        /// <summary>
        /// Moves the camera's position by the vector offset provided along it's own axes (relative to orientation).
        /// </summary>
        /// <param name="offset"></param>
        public void MoveRelative(Vector3 offset)
        {
            // Transform the axes of the relative vector by camera's local axes
            Vector3 transform = orientation * offset;

            position = position + transform;
            InvalidateView();
        }

        /// <summary>
        ///		Specifies a target that the camera should look at.
        /// </summary>
        /// <param name="target"></param>
        public void LookAt(Vector3 target)
        {
            this.Direction = target;// (target - derivedPosition);
            InvalidateView();
        }

        /// <summary>
        ///		Pitches the camera up/down counter-clockwise around it's local x axis.
        /// </summary>
        /// <param name="degrees"></param>
        public void Pitch(float degrees)
        {
            Vector3 xAxis = orientation * Vector3.UnitX;
            Rotate(xAxis, degrees);

            InvalidateView();
        }

        /// <summary>
        ///		Rolls the camera counter-clockwise, in degrees, around its local y axis.
        /// </summary>
        /// <param name="degrees"></param>
        public void Yaw(float degrees)
        {
            Vector3 yAxis;

            //if ( isYawFixed )
            //{
            //    // Rotate around fixed yaw axis
            //    yAxis = yawFixedAxis;
            //}
            //else
            //{
            // Rotate around local Y axis
            yAxis = orientation * Vector3.UnitY;
            //}

            Rotate(yAxis, degrees);

            InvalidateView();
        }

        /// <summary>
        ///		Rolls the camera counter-clockwise, in degrees, around its local z axis.
        /// </summary>
        /// <param name="degrees"></param>
        public void Roll(float degrees)
        {
            // Rotate around local Z axis
            Vector3 zAxis = orientation * Vector3.UnitZ;
            Rotate(zAxis, degrees);

            InvalidateView();
        }

        /// <summary>
        ///		Rotates the camera about an arbitrary axis.
        /// </summary>
        /// <param name="quat"></param>
        public void Rotate(Quaternion quat)
        {
            // Note the order of the multiplication
            orientation = Helpers.Vec3QuatMultiply(quat, orientation);

            InvalidateView();
        }

        /// <summary>
        ///		Rotates the camera about an arbitrary axis.
        /// </summary>
        /// <param name="axis"></param>
        /// <param name="degrees"></param>
        public void Rotate(Vector3 axis, float degrees)
        {
            Quaternion q = Quaternion.CreateFromAxisAngle(axis, MathHelper.ToRadians(degrees));
            Rotate(q);
        }

        public object Clone()
        {
            Camera c = new Camera();
            c.aspectRatio = this.aspectRatio;
            c.autoAspectRatio = this.autoAspectRatio;
            c.direction = this.direction;
            c.farDistance = this.farDistance;
            c.fieldOfView = this.fieldOfView;
            c.nearDistance = this.nearDistance;
            c.orientation = this.orientation;
            c.position = this.position;
            c.projectionMatrix = this.projectionMatrix;
            c.projectionType = this.projectionType;
            c.needsUpdate = this.needsUpdate;
            c.viewMatrix = this.viewMatrix;
            c.viewport = this.viewport;
            c.viewportChanged = this.viewportChanged;
            return c;
        }

    }
}
