
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace BasicEditor.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class GridRenderer : Microsoft.Xna.Framework.DrawableGameComponent
    {

        public enum GridType
        {
            Points,
            Lines
        }

        internal protected float gridWidth = 16;
        internal protected float gridHeight = 16;
        internal protected Editor editor;
        internal protected GridType gridStyle = GridType.Lines;
        private Color color = Color.Blue;
        private bool needsInit = true;
        private Vector3 offset;
        private VertexDeclaration decl;
        private int vertCount = 0;
        private int maxVertCount = 0;
        private VertexBuffer vb;
        private BasicEffect fx;
        private float pointSize = 4;
        private string camera;
        private Camera gridCamera;


        public string Camera
        {
            get { return camera; }
            set { camera = value; }
        }     	

        public float PointSize
        {
            get { return pointSize; }
            set { pointSize = value; }
        }         
                               

        public Vector3 Offset
        {
            get { return offset; }
            set { offset = value; }
        }

        public Color GridColor
        {
            get { return color; }
            set { color = value; }
        }

        public GridType GridStyle
        {
            get { return gridStyle; }
            set
            {
                gridStyle = value;
                needsInit = true;
            }
        }

        public float GridWidth
        {
            get { return gridWidth; }
            set
            {
                gridWidth = value;
                needsInit = true;
            }
        }

        public float GridHeight
        {
            get { return gridHeight; }
            set
            {
                gridHeight = value;
                needsInit = true;
            }
        }

        public GridRenderer(Game game)
            : base(game)
        {
            this.editor = (Editor)game;
        }

        public GridRenderer(Game game, float gw, float gh)
            : base(game)
        {
            this.editor = (Editor)game;
            this.gridWidth = gw;
            this.gridHeight = gh;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            base.Initialize();
            decl = new VertexDeclaration(this.GraphicsDevice, VertexPositionColor.VertexElements);
            fx = new BasicEffect(this.GraphicsDevice, null);
            this.gridCamera = this.editor.cam;// (Camera)this.editor.cam.Clone();
           // this.gridCamera.ProjectionType = Projection.Orthographic;
            InitGrid();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            if (this.gridCamera.NeedsUpdate) this.gridCamera.Update();
            if (needsInit) { InitGrid(); }

            base.Update(gameTime);
        }

        private void InitGrid()
        {
            int count = (this.GraphicsDevice.Viewport.Width * 2) / (int)this.gridWidth;
            if (gridStyle != GridType.Points)
            {
                count += (this.GraphicsDevice.Viewport.Height * 2) / (int)this.gridHeight;
            }
            else
            {
                count *= (this.GraphicsDevice.Viewport.Height * 2) / (int)this.gridHeight;
            }
            VertexPositionColor[] verts = new VertexPositionColor[count + 3];
            if (verts.Length > maxVertCount)
            {
                // dispose vb if prev created
                if (vb != null) { vb.Dispose(); vb = null; }
                // create new vb
                vb = new VertexBuffer(this.GraphicsDevice, verts.Length * VertexPositionColor.SizeInBytes, ResourceUsage.None, ResourceManagementMode.Automatic);
                maxVertCount = verts.Length;
            }
           
            int index = 0;
            switch (this.gridStyle)
            {
                case GridType.Lines:
                    // generate horz lines
                    for (float idy = 0; idy <= this.GraphicsDevice.Viewport.Height; idy += this.gridHeight)
                    {
                        verts[index] = new VertexPositionColor(new Vector3(0, idy, 0), this.color);
                        verts[index + 1] = new VertexPositionColor(new Vector3(this.GraphicsDevice.Viewport.Width, idy, 0), this.color);
                        index += 2;
                    }

                    // generate vert lines
                    for (float idy = 0; idy <= this.GraphicsDevice.Viewport.Width; idy += this.gridWidth)
                    {
                        verts[index] = new VertexPositionColor(new Vector3(idy, 0, 0), this.color);
                        verts[index + 1] = new VertexPositionColor(new Vector3(idy, this.GraphicsDevice.Viewport.Height, 0), this.color);
                        index += 2;
                    }
                    break;

                case GridType.Points:
                    for (float idy = 0; idy <= this.GraphicsDevice.Viewport.Height; idy += this.gridHeight)
                    {
                        for (float idx = 0; idx <= this.GraphicsDevice.Viewport.Width; idx += this.gridWidth)
                        {
                            verts[index] = new VertexPositionColor(new Vector3(idx, idy, 0), this.color);
                            index++;
                        }
                    }
                    break;
            }

            // populate vertex buffer
            vb.SetData<VertexPositionColor>(verts);
            vertCount = verts.Length;
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your draw code here
            GraphicsDevice g = this.GraphicsDevice;
            g.VertexDeclaration = decl;
            g.Vertices[0].SetSource(vb, 0, VertexPositionColor.SizeInBytes);

            fx.View = this.gridCamera.ViewMatrix;
            fx.Projection = this.gridCamera.ProjectionMatrix;
           // g.Viewport = this.editor.cam.Viewport;

            //fx.View = Matrix.CreateLookAt(Vector3.Forward * 100, Vector3.Zero, Vector3.Down);
            //fx.Projection = Matrix.CreateOrthographic(this.GraphicsDevice.Viewport.Width, this.GraphicsDevice.Viewport.Height, 1, 10000);
            Matrix baseWorld = Matrix.CreateTranslation(-this.editor.cam.Position);
            fx.World = baseWorld;// Matrix.Identity;// Matrix.CreateTranslation(-this.GraphicsDevice.Viewport.Width / 2, -this.GraphicsDevice.Viewport.Height / 2, 0);
            fx.VertexColorEnabled = true;
            fx.TextureEnabled = false;

            fx.Begin();
            foreach (EffectPass pass in fx.CurrentTechnique.Passes)
            {
                pass.Begin();
                switch (this.gridStyle)
                {
                    case GridType.Lines:
                        g.DrawPrimitives(PrimitiveType.LineList, 0, vertCount / 2);
                        break;

                    case GridType.Points:
                        g.RenderState.PointSize = this.pointSize;
                        g.DrawPrimitives(PrimitiveType.PointList, 0, vertCount);
                        break;
                }
                pass.End();
            }
            fx.End();

            base.Draw(gameTime);
        }
    }
}


