
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace BasicEditor.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class MapLoader : Microsoft.Xna.Framework.DrawableGameComponent
    {

        public enum LoadSteps
        {
            LoadFile,
            ParseTitle,
            ParseMapSize,
            ParseMapBlockSize,
            ParseTextures,
            ParseBlocks,
            FadeOut,
            Done
        }

        protected Editor editor;
        protected SpriteBatch sb;
        protected string gameFile = string.Empty;
        protected LoadSteps currentStep = LoadSteps.LoadFile;
        System.Xml.XmlDocument doc;
        protected string path;
        protected System.Collections.Specialized.StringCollection info = new System.Collections.Specialized.StringCollection();
        private float fadeValue = 1;
        private float fadeSpeed = 0.001f;
        private bool fadeOut = true;
        private Color fontColor = Color.White;
        private bool paused = false;



        public void ResetFade()
        {
            fadeValue = 1;
        }

        public bool Paused
        {
            get { return paused; }
            set { paused = value; }
        }

        public Color FontColor
        {
            get { return fontColor; }
            set { fontColor = value; }
        }

        public bool FadeOut
        {
            get { return fadeOut; }
            set { fadeOut = value; }
        }

        public float FadeSpeed
        {
            get { return fadeSpeed; }
            set { fadeSpeed = Math.Min(0, Math.Abs(value)); }
        }

        //public float FadeValue
        //{
        //    get { return FadeValue; }
        //    set { FadeValue = Math.Abs(value); }
        //}

        public MapLoader(Game game, string gameFile)
            : base(game)
        {
            // TODO: Construct any child components here
            this.editor = (Editor)game;
            this.gameFile = gameFile;
            this.path = System.IO.Path.GetFullPath(gameFile);
            this.path = System.IO.Path.GetDirectoryName(this.path);
            this.gameFile = System.IO.Path.GetFileName(gameFile);
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            sb = new SpriteBatch(editor.GraphicsDevice);
            doc = new System.Xml.XmlDocument();
            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            switch (currentStep)
            {
                case LoadSteps.LoadFile:
                    info.Add("Loading file '" + gameFile);

                    try
                    {
                        doc.Load(System.IO.Path.Combine(this.path, gameFile));
                        if (doc.DocumentElement.Name != "map") throw new System.IO.InvalidDataException("File does not appear to be a valid map file.");
                    }
                    catch (Exception)
                    {
                        throw;
                    }

                    // set the next step
                    currentStep = LoadSteps.ParseTitle;
                    break;

                case LoadSteps.ParseTitle:
                    ParseStringNode(doc.DocumentElement, "title", out editor.CurrentMap.mapTitle);

                    info.Add("Map Title: " + editor.CurrentMap.mapTitle);
                    currentStep = LoadSteps.ParseMapSize;
                    break;

                case LoadSteps.ParseMapSize:
                    ParseIntNode(doc.DocumentElement, "mapwidth", out editor.CurrentMap.mapWidth);
                    ParseIntNode(doc.DocumentElement, "mapheight", out editor.CurrentMap.mapHeight);
                    this.editor.CurrentMap.Initilize();

                    info.Add("Map Size: " + editor.CurrentMap.mapWidth + ", " + editor.CurrentMap.mapHeight);
                    currentStep = LoadSteps.ParseMapBlockSize;
                    break;

                case LoadSteps.ParseMapBlockSize:
                    ParseFloatNode(doc.DocumentElement, "blockwidth", out editor.CurrentMap.blockWidth);
                    ParseFloatNode(doc.DocumentElement, "blockheight", out editor.CurrentMap.blockHeight);

                    info.Add("Map Size: " + editor.CurrentMap.blockWidth + ", " + editor.CurrentMap.blockHeight);
                    currentStep = LoadSteps.ParseTextures;
                    break;

                case LoadSteps.ParseTextures:
                    info.Add("Parsing textures ...");
                    System.Xml.XmlNodeList texturelistItems = doc.DocumentElement.GetElementsByTagName("textures");
                    if (texturelistItems != null && texturelistItems.Count != 0)
                    {
                        ParseForTextureNodes(texturelistItems);
                    }

                    info.Add("... " + editor.gameTextures.Count + " textures loaded.");
                    currentStep = LoadSteps.ParseBlocks;
                    break;

                case LoadSteps.ParseBlocks:
                    info.Add("Parsing blocks ...");
                    ParseBlocks();
                    currentStep = LoadSteps.FadeOut;

                    break;

                case LoadSteps.FadeOut:
                    if (fadeOut && paused == false)
                    {
                        this.fadeValue -= fadeSpeed * gameTime.ElapsedGameTime.Milliseconds;
                        this.fadeValue = Math.Max(0, fadeValue);
                        if (this.fadeValue == 0)
                        {
                            currentStep = LoadSteps.Done;
                        }
                    }
                    else
                    {
                        currentStep = LoadSteps.Done;
                    }
                    break;

                case LoadSteps.Done:
                    info.Add("Done.");
                    // remove this compoent
                    this.editor.Components.Remove(this);
                    // add required components (in reverse order)
                    RectangleRenderer rr = new RectangleRenderer(editor);
                    int mw, mh;
                    mw = (int)(this.editor.CurrentMap.mapWidth * this.editor.CurrentMap.blockWidth);
                    mh = (int)(this.editor.CurrentMap.mapHeight * this.editor.CurrentMap.blockHeight);
                    rr.Rectangles.Add(new RectangleRenderer.RectangleInfo(new Rectangle(0, 0, mw, mh), Color.Red));
                    editor.Components.Add(rr);
                    editor.Components.Add(new GridRenderer(editor, editor.CurrentMap.blockWidth, editor.CurrentMap.blockHeight));
                    editor.Components.Add(new MapRenderer(editor));
                    editor.Components.Add(new BlockEditor(editor));
                    break;
            }



            base.Update(gameTime);
        }

        private void ParseForTextureNodes(System.Xml.XmlNodeList texturelistItems)
        {
            foreach (System.Xml.XmlNode texturesnode in texturelistItems)
            {
                System.Xml.XmlNodeList texturenodes = texturesnode.SelectNodes("texture");
                foreach (System.Xml.XmlNode tnode in texturenodes)
                {
                    Texture2D t;
                    string tName = string.Empty;
                    string fName = string.Empty;
                    ParseStringNode(tnode, "name", out tName);// try to get name
                    ParseStringNode(tnode, "file", out fName);// try to get filename
                    if (tName != string.Empty && fName != string.Empty)
                    {
                        string file = System.IO.Path.Combine(this.path, fName);
                        try
                        {
                            t = Texture2D.FromFile(editor.GraphicsDevice, file);
                        }
                        catch (Exception)
                        {
                            throw;
                        }
                        editor.gameTextures.Add(tName, t);
                    }
                }
            }
        }

        private void ParseBlocks()
        {
            System.Xml.XmlNodeList listItems = doc.DocumentElement.GetElementsByTagName("blocks");
            if (listItems != null)
            {
                foreach (System.Xml.XmlNode blocknode in listItems)
                {
                    System.Xml.XmlNodeList bnodes = blocknode.SelectNodes("block");
                    foreach (System.Xml.XmlNode bn in bnodes)
                    {
                        ParseBlockInfo(bn);
                    }
                }
            }
        }

        private void ParseBlockInfo(System.Xml.XmlNode blocknode)
        {
            Block block;
            int x, y;
            string tex = string.Empty;
            string type = string.Empty;
            Rectangle sourceRect;
            BlockType bt = BlockType.Platform;

            // get block type
            ParseStringNode(blocknode, "type", out type);
            bt = (BlockType)System.Enum.Parse(bt.GetType(), type, true);

            // get x/y pos for block           
            ParseIntNode(blocknode, "x", out x);
            ParseIntNode(blocknode, "y", out y);

            // get source values
            ParseIntNode(blocknode, "sourceleft", out sourceRect.X);
            ParseIntNode(blocknode, "sourcetop", out sourceRect.Y);
            ParseIntNode(blocknode, "sourcewidth", out sourceRect.Height);
            ParseIntNode(blocknode, "sourceheight", out sourceRect.Width);

            // get texture to use
            ParseStringNode(blocknode, "texture", out tex);

            // check if texture was specified in the texture list
            if (!editor.gameTextures.ContainsKey(tex))
            {
                throw new KeyNotFoundException("Texture with the name '" + tex + "' is not loaded in texture list.");
            }

            // create block
            block = Helpers.CreateBlock(bt, sourceRect, editor.gameTextures[tex]);

            // add block to map
            editor.CurrentMap.SetBlock(x, y, block);
        }

        private void ParseIntNode(System.Xml.XmlNode parentnode, string name, out int value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException("Could not find '" + name + "' element.");
            if (!int.TryParse(node.InnerText, out value))
            {
                throw new System.IO.InvalidDataException("'" + name + "' element does not appear to be a integer value.");
            }
            return;
        }

        private void ParseFloatNode(System.Xml.XmlNode parentnode, string name, out float value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException("Could not find '" + name + "' element.");
            if (!float.TryParse(node.InnerText, out value))
            {
                throw new System.IO.InvalidDataException("'" + name + "' element does not appear to be a integer value.");
            }
            return;
        }

        private void ParseStringNode(System.Xml.XmlNode parentnode, string name, out string value)
        {
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null) throw new System.IO.InvalidDataException("Could not find '" + name + "' element.");
            value = node.InnerText;
            return;
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your draw code here
            int y = this.GraphicsDevice.Viewport.Height - editor.font.Height; ;
            Color col = new Color(fontColor.R, fontColor.G, fontColor.B, (byte)(fadeValue * byte.MaxValue));
            for (int idx = info.Count - 1; idx >= 0; idx--)
            {
                editor.font.DrawText(info[idx], 0, y, col);
                y -= editor.font.Height + editor.font.LineSpacing;
            }

            base.Draw(gameTime);
        }
    }
}


