using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Reflection;

namespace BasicEditor
{
    public static class Helpers
    {
        public const int EXITCODE_BadDefaultFont = -1;

        public static T CreateWidget<T>(string file, string name) where T : Classes.GUIWidget, new()
        {
            // get assembly ref
            Assembly assembly = Assembly.GetExecutingAssembly();
            // declare list to store controls
            List<T> items;
            // load controls
            items = xFrippery.GUIReader.Load<T>(assembly.GetManifestResourceStream(file));
            // declare widget reference
            T w = null;
            // find widget, searching only top level widgets
            foreach (T e in items)
            {
                if (string.Compare(e.Name, name) == 0)
                {
                    w = e;
                    break;
                }
            }

            // check if found widget
            if (w == null)
            {
                throw new NullReferenceException("No width with the name '" + name + "' could be found in file '" + file + "'.");
            }

            // be sure to extract data info stord in the data property
            Helpers.ExtractData(w, true);
            
            // call init on widget
            w.Initilize();

            // return wodget reference
            return w;
        }

        public static void ExtractData(Classes.GUIWidget c, bool children)
        {
            string texName = string.Empty;

            // check if data contains a tecture element
            if (c.Data.IndexOf("<texture>") != -1)
            {
                // extract texture to use
                texName = Helpers.ExtractData(c.Data, "texture");

                // draw if texture exists
                if (texName != c.TextureName)
                {
                    c.TextureName = texName;
                   // c.textureNeedsUpdate = true;
                }
            }
            Rectangle sr = c.SourceRectangle;

            // extract source texture rect
            if (!int.TryParse(ExtractData(c.Data, "sourceleft"), out sr.X))
            {
                // do nothing?
            }
            if (!int.TryParse(ExtractData(c.Data, "sourcetop"), out sr.Y))
            {
                // do nothing?
            }
            if (!int.TryParse(ExtractData(c.Data, "sourcewidth"), out sr.Width))
            {
                // do nothing?
            }
            if (!int.TryParse(ExtractData(c.Data, "sourceheight"), out sr.Height))
            {
               // do nothing?
            }

            c.SourceRectangle = sr;

            if (children)
            {
                // extract data for children also
                foreach (Classes.GUIWidget child in c.Children)
                {
                   // if (child.GetType() == typeof(Interfaces.IGUIWidget))
                   // {
                       // Classes.GUIWidget gw = (GUIWidget)child;
                       // gw.ExtractData(children);
                        Helpers.ExtractData(child, children);
                    //}
                }
            }  
        }

        public static string ExtractData(string data, string e)
        {
            string tmp = "<" + e + ">";
            if (data.IndexOf(tmp) != -1)
            {
                data = data.Substring(data.IndexOf(tmp) + tmp.Length);
                data = data.Substring(0, data.IndexOf("<"));
                return data;
            }
            return string.Empty;
        }

        public static Classes.GUIWidget ConvertToWidget(xFrippery.Control c)
        {
            Classes.GUIWidget widget = new Classes.GUIWidget();
            widget.Anchor = c.Anchor;
            widget.Bounds = c.Bounds;
            widget.Children = c.Children;
            widget.Data = c.Data;
            widget.Name = c.Name;
            widget.Parent = c.Parent;
            widget.Visible = c.Visible;
            return widget;
        }

        public static Block CreateBlock(BlockType type, Rectangle source, Texture2D tex)
        {
            Block b;
            switch (type)
            {
                case BlockType.Platform:
                    b = new Classes.Platform();
                    break;

                default:
                    throw new System.IO.InvalidDataException("Invalid block type specified.");

            }
            b.SourceRectangle = source;
            b.tex = tex;
            return b;
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="quat"></param>
        /// <param name="vector"></param>
        /// <returns></returns>
        public static Vector3 Vec3QuatMultiply(Quaternion quat, Vector3 vector)
        {
            // nVidia SDK implementation
            Vector3 uv, uuv;
            Vector3 qvec = new Vector3(quat.X, quat.Y, quat.Z);
            Vector3.Cross(ref qvec, ref vector, out uv);
            // uv = qvec.Cross(vector);
            Vector3.Cross(ref qvec, ref uv, out uuv);
            // uuv = qvec.Cross(uv);
            uv *= (2.0f * quat.W);
            uuv *= 2.0f;

            return vector + uv + uuv;

            // get the rotation matrix of the Quaternion and multiply it times the vector
            //return quat.ToRotationMatrix() * vector;
        }

        /// <summary>
        /// Creates a triangle fan of 2 polys along the x/y axis centered around 0,0,0 with a width and height of 1.
        /// </summary>
        /// <returns>Returns a new vertexbuffer</returns>
        public static VertexBuffer CreateVB(GraphicsDevice d)
        {
            VertexPositionNormalTexture[] verts = new VertexPositionNormalTexture[4];
            verts[0] = new VertexPositionNormalTexture(new Vector3(-0.5f, -0.5f, 0),
                                                       new Vector3(-0.5f, -0.5f, 1),
                                                       new Vector2(1, 1));
            verts[1] = new VertexPositionNormalTexture(new Vector3(0.5f, -0.5f, 0),
                                                       new Vector3(0.5f, -0.5f, 1),
                                                       new Vector2(0, 1));
            verts[2] = new VertexPositionNormalTexture(new Vector3(0.5f, 0.5f, 0),
                                                       new Vector3(0.5f, 0.5f, 1),
                                                       new Vector2(0, 0));
            verts[3] = new VertexPositionNormalTexture(new Vector3(-0.5f, 0.5f, 0),
                                                       new Vector3(-0.5f, 0.5f, 1),
                                                       new Vector2(1, 0));

            VertexBuffer tmp = new VertexBuffer(d, 4 * VertexPositionNormalTexture.SizeInBytes,
                                                ResourceUsage.None, ResourceManagementMode.Automatic);
            tmp.SetData<VertexPositionNormalTexture>(verts);
            return tmp;
        }
    }
}
